@@ -11,6 +11,7 @@ from typing import (
     AsyncIterator,
     Awaitable,
     Callable,
+    ClassVar,
     Dict,
     Iterable,
     Iterator,
@@ -410,7 +411,7 @@ class CompletionCollector(ModelHelperMixin):
             ):
                 languages = [v for v in languages if v.code != "en"]
 
-            headers = itertools.chain(*(lang.headers.keys() for lang in languages))
+            headers = set(itertools.chain(*(lang.headers.keys() for lang in languages)))
 
         return [
             CompletionItem(
@@ -451,7 +452,7 @@ class CompletionCollector(ModelHelperMixin):
             for s in self.namespace.imports_manager.environment.keys()
         ]
 
-    _VARIABLE_COMPLETION_SORT_TEXT_PREFIX = {
+    _VARIABLE_COMPLETION_SORT_TEXT_PREFIX: ClassVar[Dict[VariableDefinitionType, str]] = {
         VariableDefinitionType.LOCAL_VARIABLE: "033",
         VariableDefinitionType.ARGUMENT: "034",
         VariableDefinitionType.VARIABLE: "035",
@@ -577,7 +578,9 @@ class CompletionCollector(ModelHelperMixin):
             for setting in settings
         ]
 
-    async def create_bdd_prefix_completion_items(self, range: Optional[Range]) -> List[CompletionItem]:
+    async def create_bdd_prefix_completion_items(
+        self, range: Optional[Range], at_top: bool = False, with_space: bool = True
+    ) -> List[CompletionItem]:
         prefixes = {"Given", "When", "Then", "And", "But"}
 
         if self.namespace.languages is not None:
@@ -588,9 +591,11 @@ class CompletionCollector(ModelHelperMixin):
                 label=prefix,
                 kind=CompletionItemKind.UNIT,
                 detail="BDD Prefix",
-                sort_text=f"080_{prefix}",
+                sort_text=f"000_{prefix}" if at_top else f"080_{prefix}",
                 insert_text_format=InsertTextFormat.PLAIN_TEXT,
-                text_edit=TextEdit(range=range, new_text=f"{prefix} ") if range is not None else None,
+                text_edit=TextEdit(range=range, new_text=prefix + (" " if with_space else ""))
+                if range is not None
+                else None,
             )
             for prefix in prefixes
         ]
@@ -673,18 +678,29 @@ class CompletionCollector(ModelHelperMixin):
 
         has_bdd = False
         bdd_token = None
+        only_bdd = False
 
         if token is not None:
             old_token = token
             bdd_token, token = self.split_bdd_prefix(self.namespace, token)
 
+            if (
+                bdd_token is None
+                and token is not None
+                and self.is_bdd_token(self.namespace, token)
+                and position.character > range_from_token(token).end.character
+            ):
+                bdd_token = token
+                token = None
+                only_bdd = True
+
             if token is not None and token.value == "":
                 token = None
 
             if bdd_token is not None and position.character > range_from_token(bdd_token).end.character:
                 has_bdd = True
 
-            if not has_bdd and token is None:
+            if not has_bdd and token is not None:
                 token = old_token
 
         if token is not None:
@@ -876,12 +892,20 @@ class CompletionCollector(ModelHelperMixin):
                 )
             )
 
-        if add_bdd_prefixes and not has_bdd:
-            result += await self.create_bdd_prefix_completion_items(
-                range_from_token(token) if token is not None else None
-            )
+        if add_bdd_prefixes and not (has_bdd or only_bdd):
+            bdd_range = r
+            at_top = False
+            with_space = True
+            if bdd_token is not None and (
+                position in range_from_token(bdd_token) or position == range_from_token(bdd_token).end.character
+            ):
+                at_top = True
+                with_space = False
+                bdd_range = range_from_token(bdd_token)
 
-        if add_reserverd:
+            result += await self.create_bdd_prefix_completion_items(bdd_range, at_top, with_space)
+
+        if add_reserverd and not (has_bdd or only_bdd):
             for k in get_reserved_keywords():
                 result.append(
                     CompletionItem(
@@ -917,6 +941,7 @@ class CompletionCollector(ModelHelperMixin):
         context: Optional[CompletionContext],
     ) -> Optional[List[CompletionItem]]:
         from robot.parsing.lexer.tokens import Token as RobotToken
+        from robot.parsing.model.blocks import SettingSection
         from robot.parsing.model.statements import Arguments, Statement
 
         if len(nodes_at_position) > 1 and isinstance(nodes_at_position[0], Statement):
@@ -942,6 +967,14 @@ class CompletionCollector(ModelHelperMixin):
                             return await self.create_headers_completion_items(r)
 
         elif position.character == 0:
+            if not nodes_at_position and position.line > 0:
+                nodes_at_line_before = await get_nodes_at_position(self.model, Position(position.line - 1, 0))
+                if nodes_at_line_before and any(isinstance(n, SettingSection) for n in nodes_at_line_before):
+                    return [
+                        *await self.create_settings_completion_items(None),
+                        *await self.create_headers_completion_items(None),
+                    ]
+
             return await self.create_headers_completion_items(None)
 
         if len(nodes_at_position) > 1 and isinstance(nodes_at_position[0], HasTokens):
@@ -1027,6 +1060,16 @@ class CompletionCollector(ModelHelperMixin):
             return None
 
         if nodes_at_position.index(node) > 0 and not isinstance(nodes_at_position[0], SectionHeader):
+            node_at_pos = nodes_at_position[0]
+            if (
+                position.character > 0
+                and isinstance(node_at_pos, HasTokens)
+                and node_at_pos.tokens
+                and node_at_pos.tokens[0].value
+                and whitespace_at_begin_of_token(node_at_pos.tokens[0]) > 0
+            ):
+                return None
+
             statement_node = cast(Statement, nodes_at_position[0])
             if len(statement_node.tokens) > 0:
                 token = cast(Token, statement_node.tokens[0])
@@ -1535,7 +1578,7 @@ class CompletionCollector(ModelHelperMixin):
 
         async def complete_arguments() -> Optional[List[CompletionItem]]:
             if (
-                import_node.name is None
+                not import_node.name
                 or position <= range_from_token(import_node.get_token(RobotToken.NAME)).extend(end_character=1).end
             ):
                 return None
@@ -1867,7 +1910,7 @@ class CompletionCollector(ModelHelperMixin):
 
         async def complete_arguments() -> Optional[List[CompletionItem]]:
             if (
-                import_node.name is None
+                not import_node.name
                 or position <= range_from_token(import_node.get_token(RobotToken.NAME)).extend(end_character=1).end
             ):
                 return None
@@ -2108,3 +2151,107 @@ class CompletionCollector(ModelHelperMixin):
             return None
 
         return await self._complete_KeywordCall_or_Fixture(RobotToken.NAME, node, nodes_at_position, position, context)
+
+    async def create_tags_completion_items(self, range: Optional[Range]) -> List[CompletionItem]:
+        built_in_tags = {
+            "robot:continue-on-failure",
+            "robot:stop-on-failure",
+            "robot:no-dry-run",
+            "robot:exit",
+        }
+
+        if get_robot_version() >= (5, 0):
+            built_in_tags.add("robot:skip")
+            built_in_tags.add("robot:skip-on-failure")
+            built_in_tags.add("robot:exclude")
+            built_in_tags.add("robot:recursive-continue-on-failure")
+
+        if get_robot_version() >= (6, 0):
+            built_in_tags.add("robot:recursive-stop-on-failure")
+            built_in_tags.add("robot:private")
+
+        if get_robot_version() >= (6, 1):
+            built_in_tags.add("robot:flatten")
+
+        return [
+            CompletionItem(
+                label=tag,
+                kind=CompletionItemKind.ENUM_MEMBER,
+                detail="Reserved Tag",
+                sort_text=f"080_{tag}",
+                insert_text_format=InsertTextFormat.PLAIN_TEXT,
+                text_edit=TextEdit(range=range, new_text=f"{tag}") if range is not None else None,
+            )
+            for tag in built_in_tags
+        ]
+
+    async def _complete_ForceTags_or_KeywordTags_or_DefaultTags_Tags(  # noqa: N802
+        self,
+        node: ast.AST,
+        nodes_at_position: List[ast.AST],
+        position: Position,
+        context: Optional[CompletionContext],
+    ) -> Union[List[CompletionItem], CompletionList, None]:
+        from robot.parsing.lexer.tokens import Token as RobotToken
+        from robot.parsing.model.statements import Statement
+
+        statement = cast(Statement, node)
+        tokens = get_tokens_at_position(statement, position)
+
+        if not tokens:
+            return None
+
+        if tokens[-1].type == RobotToken.ARGUMENT:
+            return await self.create_tags_completion_items(range_from_token(tokens[-1]))
+
+        if len(tokens) > 1 and tokens[-2].type == RobotToken.ARGUMENT:
+            return await self.create_tags_completion_items(range_from_token(tokens[-2]))
+
+        if whitespace_at_begin_of_token(tokens[-1]) >= 2:
+            return await self.create_tags_completion_items(None)
+
+        return None
+
+    async def complete_ForceTags(  # noqa: N802
+        self,
+        node: ast.AST,
+        nodes_at_position: List[ast.AST],
+        position: Position,
+        context: Optional[CompletionContext],
+    ) -> Union[List[CompletionItem], CompletionList, None]:
+        return await self._complete_ForceTags_or_KeywordTags_or_DefaultTags_Tags(
+            node, nodes_at_position, position, context
+        )
+
+    async def complete_KeywordTags(  # noqa: N802
+        self,
+        node: ast.AST,
+        nodes_at_position: List[ast.AST],
+        position: Position,
+        context: Optional[CompletionContext],
+    ) -> Union[List[CompletionItem], CompletionList, None]:
+        return await self._complete_ForceTags_or_KeywordTags_or_DefaultTags_Tags(
+            node, nodes_at_position, position, context
+        )
+
+    async def complete_DefaultTags(  # noqa: N802
+        self,
+        node: ast.AST,
+        nodes_at_position: List[ast.AST],
+        position: Position,
+        context: Optional[CompletionContext],
+    ) -> Union[List[CompletionItem], CompletionList, None]:
+        return await self._complete_ForceTags_or_KeywordTags_or_DefaultTags_Tags(
+            node, nodes_at_position, position, context
+        )
+
+    async def complete_Tags(  # noqa: N802
+        self,
+        node: ast.AST,
+        nodes_at_position: List[ast.AST],
+        position: Position,
+        context: Optional[CompletionContext],
+    ) -> Union[List[CompletionItem], CompletionList, None]:
+        return await self._complete_ForceTags_or_KeywordTags_or_DefaultTags_Tags(
+            node, nodes_at_position, position, context
+        )
